# -*- perl -*-
# Name this file News/INNAdmin.pm and be sure that it is find in @INC.
#
#
# Copyright M.C. Widerkrantz <mc@hack.org>, 2004.
# BSD license. No advertisement clause.
# Do what thou wilt. Happy hacking!

package News::INNAdmin;

use strict;
use BerkeleyDB;

sub BEGIN 
{
    use Exporter ();
    use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);

    # set the version for version checking
    $VERSION = 1.00;
    # if using RCS/CVS, this may be preferred
    $VERSION = do { my @r = (q$Revision: 1.3 $ =~ /\d+/g); sprintf "%d."."%02d" x $#r, @r }; # must be all one line, for MakeMaker

    @ISA         = qw(Exporter);

    @EXPORT = qw(&getgroup &putgroup &delgroup &listallgroups &getuser
                 &putuser &deluser &listallusers &getacl &putacl
                 &delacl &listallacls &getimport &putimport &delimport &groupflag
                 &aclflag $ng_secret $ng_acl $acl_read $acl_post
                 $acl_approve $acl_external);

    %EXPORT_TAGS = ( );     # eg: TAG => [ qw!name1 name2! ],

    # your exported package globals go here,
    # as well as any optionally exported functions
    # @EXPORT_OK   = qw($Var1 %Hashit &func3);

    @EXPORT_OK =  qw($newsserver $dbprefix $usersdb $importsdb $groupsdb
		     $aclprefix $htpasswd $mailhost $gateuser $gatepass $adminbase);

    my $CONFIGFILE = 'innadmin.conf';
    
    if (-f  $CONFIGFILE)
    {
	do $CONFIGFILE;
    }
    else
    {
	print "Cannot find configuration file $CONFIGFILE. Cannot continue.\n";
	exit 1;
    }
}

use vars @EXPORT, @EXPORT_OK;

# non-exported package globals go here
#use vars      qw(@more $stuff);

# use vars qw($newsserver $dbprefix $usersdb $importsdb $groupsdb
#             $aclprefix $htpasswd $mailhost $gateuser $gatepass $adminbase);

### initialize package globals, first exported ones

# Flags for the newsgroup access. More may come.

$ng_secret = 1;
$ng_acl    = 1 << 1;

# Flags for access control.

$acl_read     = 1;
$acl_post     = 1 << 1;
$acl_approve  = 1 << 2;
$acl_external = 1 << 3;

### then the others (which are still accessible as $Some::Module::stuff)
#$stuff  = '';
#@more   = ();

# all file-scoped lexicals must be created before
# the functions below that use them.

# file-private lexicals go here
#my $priv_var    = '';
#my %secret_hash = ();

# here's a file-private function as a closure,
# callable as &$priv_func;  it cannot be prototyped.
#my $priv_func = sub {
    # stuff goes here.
#};

# @value = getgroup(groupname)
sub getgroup
{
    my ($groupname) = @_;
    my %groups;
    my @value;

    unless (tie(%groups, "BerkeleyDB::Hash", -Filename => $groupsdb, 
                -Flags => DB_CREATE, -Mode => 0644))
    {
        print "Cannot open file $groupsdb: $! $BerkeleyDB::Error\n";
        return 0;
    }

    if (!$groups{$groupname})
    {
	untie %groups;
	return 0;
    }

    @value = split(/\0/, $groups{$groupname});

    untie %groups;

    return @value;
}

# putgroup($groupname, @values)
sub putgroup
{
    my ($groupname, @value) = @_;
    my %groups;

    unless (tie(%groups, "BerkeleyDB::Hash", -Filename => $groupsdb, 
                -Flags => DB_CREATE, -Mode => 0644))
    {
        print "Cannot open file $groupsdb: $! $BerkeleyDB::Error\n";
        return 0;
    }

    $groups{$groupname} = join("\0", @value, "\0");

    untie %groups;
}

# delgroup($groupname)
sub delgroup
{
    my ($groupname) = @_;
    my %groups;

    unless (tie(%groups, "BerkeleyDB::Hash", -Filename => $groupsdb, 
                -Flags => DB_CREATE, -Mode => 0644))
    {
        print "Cannot open file $groupsdb: $! $BerkeleyDB::Error\n";
        return 0;
    }

    delacltable($groupname);
    delete $groups{$groupname};

    untie %groups;
}

sub listallgroups
{
   my %groups;
   my %allgroups;
   my @value;
   my ($k, $v);
   my $dbname = $groupsdb;

   unless (tie(%groups, "BerkeleyDB::Hash", -Filename => $dbname, 
	       -Flags => DB_CREATE, -Mode => 0644))
   {
       print "Cannot open file $dbname: $! $BerkeleyDB::Error\n";
       return 0;
   }

   %allgroups = %groups;

   untie %groups;
   return %allgroups;
}

# @value = getuser(username)
sub getuser
{
    my ($username) = @_;
    my %users;
    my @value;

    unless (tie(%users, "BerkeleyDB::Hash", -Filename => $usersdb, 
                -Flags => DB_CREATE, -Mode => 0644))
    {
        print "Cannot open file $usersdb: $! $BerkeleyDB::Error\n";
        return 0;
    }

    if (!$users{$username})
    {
	untie %users;
	return 0;
    }

    @value = split(/\0/, $users{$username});
    untie %users;
    return @value; 
}

sub listallusers
{
   my %users;
   my %allusers;
   my @value;
   my ($k, $v);

   unless (tie(%users, "BerkeleyDB::Hash", -Filename => $usersdb, 
	       -Flags => DB_CREATE, -Mode => 0644))
   {
       print "Cannot open file $usersdb: $! $BerkeleyDB::Error\n";
       return 0;
   }

   %allusers = %users;

   untie %users;
   return %allusers;
}

# putuser($username, @value)

sub putuser
{
    my ($username, @value) = @_;

    my %users;

    unless (tie(%users, "BerkeleyDB::Hash", -Filename => $usersdb, 
                -Flags => DB_CREATE, -Mode => 0644))
    {
        print "Cannot open file $usersdb: $! $BerkeleyDB::Error\n";
        return 0;
    }

    # The \0 at the end is because we want a finishing NUL as well.

    $users{$username} = join("\0", @value, "\0");

    untie %users;
}

# deluser($username)
sub deluser
{
    my ($username) = @_;
    my %users;

    unless (tie(%users, "BerkeleyDB::Hash", -Filename => $usersdb, 
                -Flags => DB_CREATE, -Mode => 0644))
    {
        print "Cannot open file $usersdb: $! $BerkeleyDB::Error\n";
        return 0;
    }

    delete $users{$username};

    untie %users;
}

# aclflag = getacl($groupname, $address)
sub getacl
{
    my ($groupname, $address) = @_;
    my $dbname = "$aclprefix/$groupname";
    my %acls;
    my $access;

    unless (tie(%acls, "BerkeleyDB::Hash", -Filename => $dbname, 
                -Flags => DB_CREATE, -Mode => 0644))
    {
        print "Cannot open file $dbname: $! $BerkeleyDB::Error\n";
        return 0;
    }

    if (!$acls{$address})
    {
	untie %acls;
	return 0;
    }

    $access = $acls{$address};
    untie %acls;
    return $access;
}

# putacl($newsgroup, $address, $access)
sub putacl
{
    my ($groupname, $address, $access) = @_;
    my $dbname = "$aclprefix/$groupname";
    my %acls;

    unless (tie(%acls, "BerkeleyDB::Hash", -Filename => $dbname, 
                -Flags => DB_CREATE, -Mode => 0644))
    {
        print "Cannot open file $dbname: $! $BerkeleyDB::Error\n";
        return 0;
    }

    $acls{$address} = $access;
    untie %acls; 
}

# delacl($groupname, $address)
# return false if there is none.
sub delacl
{
    my ($groupname, $address) = @_;
    my $dbname = "$aclprefix/$groupname";
    my %acls;

    unless (tie(%acls, "BerkeleyDB::Hash", -Filename => $dbname, 
                -Flags => DB_CREATE, -Mode => 0644))
    {
        print "Cannot open file $dbname: $! $BerkeleyDB::Error\n";
        return 0;
    }

    if (!$acls{$address})
    {
	untie %acls; 
	return 0;
    }
    
    delete $acls{$address};
    untie %acls;
    return 1;
}

# listallacls($groupname)
sub listallacls
{
    my ($groupname) = @_;
    my %acls;
    my %allacls;    
    my $flags;
    my ($k, $v);
    my $dbname = "$aclprefix/$groupname";

    unless (tie(%acls, "BerkeleyDB::Hash", -Filename => $dbname, 
		-Flags => DB_CREATE, -Mode => 0644))
    {
	print "Cannot open file $dbname: $! $BerkeleyDB::Error\n";
	return 0;
    }

    %allacls = %acls;

    untie %acls;
    return %allacls;
}

# delacltable($groupname)
sub delacltable
{
    my ($groupname) = @_;

    # Is this group ACL controlled?

    my ($creator, $moderator, $expire, $flags, $title,
	$description) = split(/\0/, getgroup($groupname));

    if ($flags & $ng_acl)
    {
	# Remove ACL table.
	unlink("$aclprefix/$groupname");
    }
}

# $groupname = getimport($address)
sub getimport
{
    my ($address) = @_;
    my $groupname;
    my %imports;
    my $dbname = $importsdb;

    unless (tie(%imports, "BerkeleyDB::Hash", -Filename => $dbname, 
                -Flags => DB_CREATE, -Mode => 0644))
    {
        print "Cannot open file $dbname: $! $BerkeleyDB::Error\n";
        return 0;
    }    

    if (!$imports{$address})
    {
	untie %imports;
	return 0;
    }

    $groupname = $imports{$address};
    untie %imports;    
    return $groupname;
}

# putimport($address, $groupname)
sub putimport
{
    my ($address, $groupname) = @_;
    my %imports;
    my $dbname = $importsdb;

    unless (tie(%imports, "BerkeleyDB::Hash", -Filename => $dbname, 
                -Flags => DB_CREATE, -Mode => 0644))
    {
        print "Cannot open file $dbname: $! $BerkeleyDB::Error\n";
        return;
    }    

    $imports{$address} = $groupname;

    untie %imports;
}

# delimport($address)
sub delimport
{
    my ($address) = @_;
    my %imports;
    my $dbname = $importsdb;

    unless (tie(%imports, "BerkeleyDB::Hash", -Filename => $dbname, 
                -Flags => DB_CREATE, -Mode => 0644))
    {
        print "Cannot open file $dbname: $! $BerkeleyDB::Error\n";
        return;
    }    

    delete $imports{$address};

    untie %imports;
}

# bool = groupflag(groupname, flag)
sub groupflag
{
    my ($groupname, $flag) = @_;

    my @value = getgroup($groupname);
    my ($creator, $moderator, $expire, $flags, $title,
        $description) = @value;

    if ($flag & $flags)
    {
        return -1;
    }
    else
    {
        return 0;
    }
}

# bool = aclflag(groupname, username, flag)
sub aclflag
{
    my ($groupname, $username, $flag) = @_;

    my $aclflags = getacl($groupname, $username);

    if ($flag & $aclflags)
    {
        return -1;
    }
    else
    {
        return 0;
    }
}

# this one isn't exported, but could be called!
# sub func4(\%)  {}    # proto'd to 1 hash ref

# module clean-up code here (global destructor)
sub END 
{ 

}       
