#! /usr/bin/perl
# -*- perl -*-
#
# Copyright M.C. Widerkrantz <mc@hack.org>, 2004.
# BSD license. No advertisement clause.
# Do what thou wilt. Happy hacking!

use strict;
use News::INNAdmin;
use CGI qw/:all/;

$| = 1; # Flush output at once.

# The URL to ourself
my $programurl = "naf.cgi";

# Cascading Stylesheet file to define our looks
my $styleurl = "webnews.css";

# The file containing a menu index for calls to naf.
my $menuurl = "nafindex.html";

# Our administrative user with rights to bypass every access control.

my $adminuser = "root";

# An .htpasswd file.

my $htpasswd = $News::INNAdmin::htpasswd;

# We trust the HTTP server to ask the user to login. Set appropriate
# authentication with the HTTP server, perhaps in an .htpasswd file or
# whatever.

my $user = $ENV{'REMOTE_USER'};

unless ($user)
{
    print header, start_html("Error"),
          h1("Error"),
	  p("Please identify yourself."),
          end_html;
    exit;
}


### The actual commands. ###

sub CMDlistuser
{
    my %allusers;
    my ($k, $v);

    # Say hello to the client.
    print header;

    print start_html(-title=>"News Users", -style=>{-src=>$styleurl});
    print h1("News Users");
    print p("<a href=\"$programurl\">Main menu</a>.");
    print p("<a href=\"$programurl?command=edituser\">Add new user</a>.");

    print "<table>";
    print Tr(th(['Username', 'Full name','Address']));

    %allusers = listallusers();

    while (($k, $v) = each %allusers)
    {
        my @value = split(/\0/, $v);

        my ($pass, $fullname, $address, $currgroup, $online_since,
            $last_online, $total_time, $written, $read, $description,
            $faceurl) = @value;

	print Tr(td(["<A HREF=\"$programurl?command=edituser&user=$k\">$k</A>", 
		     $fullname,
		     $address]));
    }    

    print "</table>";
    print end_html;
}

sub CMDedituser
{
    my $username = param('user');
    my ($pass, $fullname, $address, $currgroup, $online_since,
	$last_online, $total_time, $written, $read, $description,
	$faceurl);

    # Say hello to the client.
    print header;

    # Check for access rights.

    if ($user ne $username && $user ne $adminuser)
    {
	print start_html(-title=>"Permission Denied", -style=>{-src=>$styleurl});	
	print h1("Permission Denied");
	print p("<a href=\"$programurl\">Main menu</a>.");
	print p("You don't have access to edit other user's data.");
	print end_html;	
	return;
    }

    print start_html(-title=>"User Settings", -style=>{-src=>$styleurl});
    print h1("User Settings");
    print p("<a href=\"$programurl\">Main menu</a>.");
    print start_form(-method=>'POST', -action=>"$programurl");

    if (!$username)
    {
	# New user being defined.
	print "Username: ", textfield(-name=>'user', -size=>30, -maxlength=>127),p;

    }
    else
    {
	($pass, $fullname, $address, $currgroup, $online_since,
	 $last_online, $total_time, $written, $read, $description,
	 $faceurl) = getuser($username);
    
	print h2("Settings for $username");
	print hidden(-name=>'user', -value=>$username);
    }

    print "Full name: ", textfield(-name=>'fullname', -default=>$fullname, 
				   -size=>30, -maxlength=>127),p,
    "E-mail address: ", textfield(-name=>'address', -default=>$address, 
                                  -size=>30, -maxlength=>127),p,
    "Password: ", textfield(-name=>'pass', -default=>$pass, -size=>30, 
                            -maxlength=>127),p,
    submit(-name=>'command', -value=>'storeuser'),
    submit(-name=>'command', -value=>'deluser'),
    end_form;
}

sub CMDstoreuser
{
    my $username = param('user');
    my ($pass, $fullname, $address, $currgroup, $online_since,
	$last_online, $total_time, $written, $read, $description,
	$faceurl);

    $pass = param('pass');
    $fullname = param('fullname');
    $address = param('address');
    $currgroup = "";
    $online_since = 0;
    $last_online = 0;
    $total_time = 0;
    $written = 0;
    $read = 0;
    $description = "";
    $faceurl = "";

    if (!$username)
    {
	print header;
	print start_html(-title=>"Storing Error", -style=>{-src=>$styleurl});
	print h1("Storing Error");
	print p("<a href=\"$programurl\">Main menu</a>.");
	print p("No username defined. Cannot store data.");
	print end_html;
	return;
    }

    # Check for access.

    if ($user ne $username && $user ne $adminuser)
    {
	print start_html(-title=>"Permission Denied", -style=>{-src=>$styleurl});	
	print h1("Permission Denied");
	print p("<a href=\"$programurl\">Main menu</a>.");
	print p("You don't have access to edit other user's data.");
	print end_html;	
	return;
    }

    # Dump the new entry in .htpasswd for WebNews access.

    my @alphabet = ('.', '/', 0..9, 'A'..'Z', 'a'..'z');
    my $salt = join '', @alphabet[rand 64, rand 64];
    my $cryptpass = crypt ($pass, $salt);

    # FIXME: Weed the old password out if there is one.

    unless (open(HTPASS, ">>$htpasswd"))
    {
	print start_html(-title=>"Can't Open File", -style=>{-src=>$styleurl});	
	print h1("Can't Open File");
	print p("<a href=\"$programurl\">Main menu</a>.");
	print p("Can't open $htpasswd: $!\n");
	print end_html;	
	return;
    }

    print HTPASS "$username:$cryptpass\n";
    close(HTPASS);

    putuser($username, $pass, $fullname, $address, $currgroup,
	    $online_since, $last_online, $total_time, 
	    $written, $read, $description, $faceurl);

    print redirect(-uri=>"$programurl?command=listusers");
}

# CMDdeluser($username)
sub CMDdeluser
{
    my ($username) = param('user');

    # Check for access.
    if ($user ne $username && $user ne $adminuser)
    {
	print start_html(-title=>"Permission Denied", 
			 -style=>{-src=>$styleurl});	
	print h1("Permission Denied");
	print p("<a href=\"$programurl\">Main menu</a>.");
	print p("You don't have access to delete another user's data.");
	print end_html;	
	return;
    }

    deluser($username);
    print redirect(-uri=>"$programurl");
}

sub CMDlistgroups
{
    my %allgroups;
    my ($k, $v);

    # Say hello to the client.
    print header;

    print start_html(-title=>"Newsgroups", -style=>{-src=>$styleurl});
    print h1("Newsgroups");
    print p("<a href=\"$programurl\">Main menu</a>.");
    print p("<a href=\"$programurl?command=editgroup\">Add a new newsgroup</a>.");

    print "<table>";

    if ($user eq $adminuser)
    {
	print Tr(th(['Newsgroup', 'Owner', 'Expires after', 
		     'Moderator', 'ACL', 'Secret']));
    }
    else
    {
	print Tr(th(['Newsgroup', 'Expires after', 
		     'Moderator', 'ACL', 'Secret']));
    }

    %allgroups = listallgroups();

    while (($k, $v) = each %allgroups)
    {
        my @value = split(/\0/, $v);

	my ($creator, $moderator, $expire, $flags, $title,
	    $description) = @value;

	my ($aclstr, $secretstr);

	if ($flags & $ng_acl)
	{
	    $aclstr = "A";
	}

	if ($flags & $ng_secret)
	{
	    $secretstr = "S";
	}

	if ($user eq $creator || $user eq $adminuser)
	{
	    if ($user eq $adminuser)
	    {
		print Tr(td(["<A HREF=\"$programurl?command=editgroup&newsgroup=$k\">$k</A>", 
			     $creator, 
			     $expire, 
			     $moderator,
			     $aclstr,
			     $secretstr]));
	    }
	    else
	    {
		print Tr(td(["<A HREF=\"$programurl?command=editgroup&newsgroup=$k\">$k</A>", 
			     $expire, 
			     $moderator,
			     $aclstr,
			     $secretstr]));		
	    }
	}
    }    

    print "</table>"; 
    print end_html;
}

sub CMDeditgroup
{
    my $groupname = param('newsgroup');
    my ($creator, $moderator, $expire, $flags, $title,
	$description);
    my ($aclstr, $secretstr);

    $aclstr = $secretstr = 0;

    # Say hello to the client.
    print header;

    print start_html(-title=>"Newsgroup Settings", -style=>{-src=>$styleurl});
    print h1("Newsgroup Settings");
    print p("<a href=\"$programurl\">Main menu</a>");

    print start_form(-method=>'POST', -action=>"$programurl");

    # FIXME: Policy for creating new groups?

    if (!$groupname)
    {
	print "Newsgroup: ", textfield(-name=>'newsgroup', -size=>30, -maxlength=>127), p;

    }
    else
    {

	($creator, $moderator, $expire, $flags, $title,
	 $description) = getgroup($groupname);

	# Check access.

	if ($user ne $creator && $user ne $adminuser)
	{
	    print p("You don't have access to edit this newsgroup.");
	    print end_html;
	    return;
	}
	
	print h2("Settings for $groupname");
	print hidden(-name=>'newsgroup', -value=>$groupname);

    }
    
    print "Owner: ", textfield(-name=>'creator', -default=>$creator,
			       -size=>30),p,
    "Moderator Address: ", textfield(-name=>'moderator', -default=>$moderator, 
				     -size=>30),p,
    "Expire after: ", textfield(-name=>'expire', -default=>$expire, -size=>3), p;

    # Flags.

    if ($flags & $ng_acl)
    {
	$aclstr = 1;
    }
    
    if ($flags & $ng_secret)
    {
	$secretstr = 1;
    }

    print checkbox('aclflag', $aclstr, "on", 'access control');
    print checkbox('secretflag', $secretstr, "on", 'secret'), p;

    print submit(-name=>'command', -value=>'storegroup'),
          submit(-name=>'command', -value=>'delgroup'),
          end_form;

    if ($flags & $ng_acl)
    {
	print p("<a href=\"$programurl?command=listacl&newsgroup=$groupname\">List access lists</a>");
    }

    print end_html;

}

sub CMDstoregroup
{
    my $groupname = param('newsgroup');
    my ($creator, $moderator, $expire, $flags, $title,
	$description);

    $creator = param('creator');
    $moderator = param('moderator');
    $expire = param('expire');
    $flags = 0;
    $title = "";
    $description = "";

    my $aclstr = param('aclflag');
    my $secretstr = param('secretflag');

    if ($aclstr)
    {
	$flags += $ng_acl;
    }

    if ($secretstr)
    {
	$flags += $ng_secret;
    }


    if (!$groupname)
    {
	print header;
	print start_html(-title=>"Storing Error", -style=>{-src=>$styleurl});	
	print h1("Storing Error");
	print p("<a href=\"$programurl\">Main menu</a>.");
	print p("No newsgroup name defined. Cannot store data.");
	print end_html;
	return;
    }

    # Check access.

    if (!getgroup($groupname))
    {
	# A new group.

	# FIXME: Policy for creating new groups?
    }
    else
    {
	my ($creator, $moderator, $expire, $flags, $title,
	    $description) = getgroup($groupname);

	if ($user ne $creator && $user ne $adminuser)
	{
	    print p("You don't have access to edit this newsgroup.");
	    print end_html;
	    return;
	}
    }

    if (! $moderator)
    {
        system("ctlinnd newgroup $groupname y $creator");
    }
    else
    {
        system("ctlinnd newgroup $groupname m $creator");
        # FIXME: Insert moderator into moderators file.
    }

    putgroup($groupname, $creator, $moderator, $expire, $flags, $title,
	     $description);

    print redirect(-uri=>"$programurl?command=listgroups");
}

# CMDdelgroup($groupname)
sub CMDdelgroup
{
    my ($groupname) = param('newsgroup');

    # Check for access.

    my ($creator, $moderator, $expire, $flags, $title, $description) =
	getgroup($groupname);

    if ($user ne $creator && $user ne $adminuser)
    {
	print start_html(-title=>"Permission Denied", 
			 -style=>{-src=>$styleurl});	
	print h1("Permission Denied");
	print p("<a href=\"$programurl\">Main menu</a>.");
	print p("You don't have access to delete another user's data.");
	print end_html;	
	return;
    }

    delgroup($groupname);
}

sub CMDeditacl()
{
    my $address = param('address');
    my $groupname = param('newsgroup');
    my $aclflags;

    # Say hello to the client.
    print header;

    print start_html(-title=>"Access Control Settings", -style=>{-src=>$styleurl});
    print h1("Access Control Settings");
    print p("<a href=\"$programurl\">Main menu</a>.");

    print start_form(-method=>'POST', -action=>"$programurl");

    if (!$groupname)
    {
	print "Newsgroup: ", textfield(-name=>'newsgroup', -size=>30, -maxlength=>127), p;
    }
    else
    {
	print hidden(-name=>'newsgroup', -value=>$groupname);
	print p("Adding a new member for $groupname.");
    }
    if (!$address)
    {
	print "E-mail Address: ", textfield(-name=>'address', -size=>30, -maxlength=>127), p;
    }
    else
    {
	print hidden(-name=>'address', -value=>$address);
	print p("Adding $address to a newsgroup.");
    }

    print checkbox('acl_read', 0, 'on', 'read'), p;
    print checkbox('acl_post', 0, 'on', 'post'), p;
    print checkbox('acl_approve', 0, 'on', 'approve'), p;
    print checkbox('acl_external', 0, 'on', 'external'), p;

    print hidden(-name=>'redirect', -value=>"$programurl?command=listacl&newsgroup=$groupname");
    print submit(-name=>'command', -value=>'storeacl'),
          submit(-name=>'command', -value=>'delacl'),
          end_form,
          end_html;
}

sub CMDstoreacl()
{
    my $address = param('address');
    my $groupname = param('newsgroup');
    my $a_read = param('acl_read');
    my $a_post = param('acl_post');
    my $a_approve = param('acl_approve');
    my $a_external = param('acl_external');
    my $redirect = param('redirect');

    if (!$address || !$groupname)
    {
	print header;
	print start_html(-title=>"Storing Error", -style=>{-src=>$styleurl});	
	print h1("Storing Error");
	print p("<a href=\"$programurl\">Main menu</a>.");
	print p("Missing data.");
	print end_html;
	return;	
    }

    my $aclflags = 0;

    if ($a_read)
    {
	$aclflags += $acl_read;
    }
    if ($a_post)
    {
	$aclflags += $acl_post;
    }
    if ($a_approve)
    {
	$aclflags += $acl_approve;
    }
    if ($a_external)
    {
	$aclflags += $acl_external;
    }

    # Check access.
    my ($creator, $moderator, $expire, $flags, $title,
	$description) = getgroup($groupname);

    if ($user ne $creator && $user ne $adminuser)
    {
	print header;
	print start_html(-title=>"Permission Denied", -style=>{-src=>$styleurl});	
	print h1("Permission Denied");
	print p("<a href=\"$programurl\">Main menu</a>.");
	print p("You don't have access to edit this access control.");	
    }

    putacl($groupname, $address, $aclflags);
    print redirect(-uri=>"$redirect");
}

sub CMDlistacl()
{
    my $groupname = param('newsgroup');
    my ($k, $v);
    my %acls;
    my ($readstr, $poststr, $approvestr, $extstr);

    $readstr = $poststr = $approvestr = $extstr = "no";

    # Say hello to the client.
    print header;

    if (!$groupname)
    {
	print start_html(-title=>"Error", -style=>{-src=>$styleurl});	
	print h1("Error");
	print p("<a href=\"$programurl\">Main menu</a>.");
	print p("No newsgroup name.");
    }

    if (!getgroup($groupname))
    {
	    print start_html(-title=>"No Group", -style=>{-src=>$styleurl});
	    print h1("No Group");

	    print p("There is no such newsgroup.");
	    print end_html;
	    return;
    }

    # Check access.

    my ($creator, $moderator, $expire, $flags, $title,
	$description) = getgroup($groupname);

    if ($user ne $creator && $user ne $adminuser)
    {
	if ($flags & $ng_secret)
	{
	    print start_html(-title=>"No Group", -style=>{-src=>$styleurl});
	    print h1("No Group");

	    print p("There is no such newsgroup.");
	    print end_html;
	    return;	    
	}
	else
	{
	    print start_html(-title=>"Permission Denied", -style=>{-src=>$styleurl});
	    print h1("Permission Denied");

	    print p("You have no right to edit access control lists for this group.");
	    print end_html;
	    return;
	}
    }
    
    %acls = listallacls($groupname);

    print start_html(-title=>"$groupname ACL", -style=>{-src=>$styleurl});
    print h1("$groupname ACL");
    print p("<a href=\"$programurl\">Main menu</a>.");
    print p("<a href=\"$programurl?command=editacl&newsgroup=$groupname\">Add new member</a>.");

    print "<table>";
    print Tr(th(['Address', 'Read', 'Post', 'Approve', 'External']));

    while (($k, $v) = each %acls)
    {
	if ($v & $acl_read)
	{
	    $readstr = "yes";
	}
	if ($v & $acl_post)
	{
	    $poststr = "yes";
	}
	if ($v & $acl_approve)
	{
	    $approvestr = "yes";
	}
	if ($v & $acl_external)
	{
	    $extstr = "yes";
	}

	print Tr(td(["$k", 
		     $readstr, 
		     $poststr, 
		     $approvestr,
		     $extstr]));    
    } # while

    print "</table>";
    print end_html;	
}

sub CMDdelacl
{
    my $address = param('address');
    my $groupname = param('newsgroup');    

    # Check access.
    my ($creator, $moderator, $expire, $flags, $title,
	$description) = getgroup($groupname);

    if ($user ne $creator && $user ne $adminuser)
    {
	print header;
	print start_html(-title=>"Permission Denied", -style=>{-src=>$styleurl});	
	print h1("Permission Denied");
	print p("<a href=\"$programurl\">Main menu</a>.");
	print p("You don't have access to delete this access control.");
    }

    delacl($groupname, $address);
}

sub CMDstoreimport
{
    my ($address) = param('address');
    my ($groupname) = param('newsgroup');

    # Check access.
    my ($creator, $moderator, $expire, $flags, $title,
	$description) = getgroup($groupname);

    if ($user ne $creator && $user ne $adminuser)
    {
	print header;
	print start_html(-title=>"Permission Denied", -style=>{-src=>$styleurl});	
	print h1("Permission Denied");
	print p("<a href=\"$programurl\">Main menu</a>.");
	print p("You don't have access to add an import to this newsgroup.");
    }

    putimport($address, $groupname);
}

sub CMDdelimport
{
    my ($address) = param('address');
    my ($groupname) = param('newsgroup');

    # Check access.
    my ($creator, $moderator, $expire, $flags, $title,
	$description) = getgroup($groupname);

    if ($user ne $creator && $user ne $adminuser)
    {
	print header;
	print start_html(-title=>"Permission Denied", -style=>{-src=>$styleurl});	
	print h1("Permission Denied");
	print p("<a href=\"$programurl\">Main menu</a>.");
	print p("You don't have access to delete an import to this newsgroup.");
    }

    delimport($address, $groupname);
}


### Main stuff ###

# Now, what command did we receive?

my $command = param('command');

unless ($command)
{
    print redirect(-uri=>"$menuurl");
}

# User commands.

if ($command eq "listusers")
{
    CMDlistuser();
}
elsif ($command eq "edituser")
{
    CMDedituser();
}
elsif ($command eq "storeuser")
{
    CMDstoreuser();
}
elsif ($command eq "deluser")
{
    CMDdeluser();
}

# Group commands.

elsif ($command eq "listgroups")
{
    CMDlistgroups();
}
elsif ($command eq "editgroup")
{
    CMDeditgroup();
}
elsif ($command eq "storegroup")
{
    CMDstoregroup();
}
elsif ($command eq "delgroup")
{
    CMDdelgroup();
}

# Import commands, used by groups.
elsif ($command eq "storeimport")
{
    CMDstoreimport();
}
elsif ($command eq "delimport")
{
    CMDdelimport();
}

# ACL commands, used by users?
elsif ($command eq "editacl")
{
    CMDeditacl();
}
elsif ($command eq "storeacl")
{
    CMDstoreacl();
}
elsif ($command eq "listacl")
{
    CMDlistacl();
}
elsif ($command eq "delacl")
{
    CMDdelacl();
}
